EARTH_RADIUS_KM = 6371.0088
EARTH_RADIUS_MI = 3958.7613
EARTH_RADIUS_NM = 3440.0695
EARTH_RADIUS_M = 6371008.8

EARTH_SEMI_MAJOR_AXIS_M = 6378137.0
EARTH_SEMI_MINOR_AXIS_M = 6356752.314245
EARTH_FLATTENING = 1 / 298.257223563

KM_TO_MI = 0.621371
KM_TO_NM = 0.539957
MI_TO_KM = 1.609344
NM_TO_KM = 1.852
M_TO_FT = 3.28084
FT_TO_M = 0.3048

DEFAULT_CRUISE_SPEED_KTS = 450.0
DEFAULT_CO2_KG_PER_PAX_KM = 0.115
DEFAULT_MAX_LEG_KM = 5000.0
DEFAULT_ROUTING_GRAPH_DISTANCE_THRESHOLD_KM = 5000.0

VALID_AIRPORT_TYPES = {
    "small_airport",
    "medium_airport",
    "large_airport",
    "seaplane_base",
    "heliport",
    "closed",
    "balloonport"
}
